package me.unurled.raxen;

import static me.unurled.raxen.utils.Utils.colorComp;

import com.comphenix.protocol.ProtocolLibrary;
import com.comphenix.protocol.ProtocolManager;
import java.util.logging.Logger;
import lombok.Getter;
import me.unurled.raxen.config.Config;
import me.unurled.raxen.config.PlayerConfig;
import me.unurled.raxen.manager.Manager;
import me.unurled.raxen.utils.Reload;
import org.bukkit.Bukkit;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

public final class Raxen extends JavaPlugin {

  private static final String prefix =
    "<aqua>Rx</aqua><light_purple>></light_purple> ";

  @Getter
  private static String version = "0.5.7";

  private boolean loading = true;
  private final PluginManager pm = getServer().getPluginManager();

  private static Raxen plugin;

  private static Logger logger;

  private static Config config;

  private static PlayerConfig playerConfig;

  @Getter
  private ProtocolManager protocolManager;

  @Getter
  private Manager manager;

  /**
   * Method when the server launch the plugin.
   */
  @Override
  public void onEnable() {
    plugin = this;
    logger = getLogger();

    version = this.getPluginMeta().getVersion();

    manager = new Manager();
    manager.set(plugin);

    // Config
    config = new Config(plugin);

    protocolManager = ProtocolLibrary.getProtocolManager();

    playerConfig = new PlayerConfig(plugin);

    //register Commands and Events
    registerCommands();
    registerEvents();

    Bukkit
      .getConsoleSender()
      .sendMessage(
        colorComp(prefix + "<green>Server Started Successfully!</green>")
      );

    manager.getListenerManager().getServerPingEvent().setLoading(false);
    loading = false;
  }

  /**
   * register the command in the command manager class.
   */
  private void registerCommands() {
    manager.getCommandManager().register();

    Bukkit
      .getConsoleSender()
      .sendMessage(colorComp(prefix + "<green>Commands Registered!</green>"));
  }

  /**
   * register the events in the event manager class.
   */
  private void registerEvents() {
    manager.getListenerManager().register();

    Bukkit
      .getConsoleSender()
      .sendMessage(colorComp(prefix + "<green>Events Registered!</green>"));
  }

  /**
   * Method when the server stops the plugin.
   */
  @Override
  public void onDisable() {
    if (!loading) {
      manager.getListenerManager().getServerPingEvent().setLoading(true);
      Reload.kickAll();

      playerConfig.close();
    }

    Bukkit
      .getConsoleSender()
      .sendMessage(
        colorComp(prefix + "<red>Server Stopped Successfully!</red>")
      );
  }

  /**
   * getter for the logger
   * @return the plugin logger
   */
  public Logger getPluginLogger() {
    return logger;
  }

  /**
   * getter for the bukkit's plugin manager
   * @return plugin manager
   */
  public PluginManager getPm() {
    return pm;
  }

  /**
   * return the instance which is launched.
   * @return main instance
   */
  public Raxen getPlugin() {
    return plugin;
  }

  /**
   * getter for the raxen prefix
   * @return string prefix
   */
  public static String getPrefix() {
    return prefix;
  }

  /**
   * getter for the yml's player configs.
   * @return player config instance
   */
  public PlayerConfig getPlayerConfig() {
    return playerConfig;
  }

  /**
   * getter for an instance of the config utility class
   * @return raxen's config
   */
  public Config getConf() {
    return config;
  }

  public void disable() {
    this.setEnabled(false);
  }
}
