package me.unurled.raxen.components.entity.Attributes;

import static me.unurled.raxen.utils.Utils.debug;

import de.tr7zw.nbtapi.NBTItem;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Entity;
import org.bukkit.entity.LivingEntity;
import org.bukkit.inventory.EntityEquipment;
import org.bukkit.inventory.ItemStack;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;

public class Attribute {

  /**
   * The NamespaceKey register for the PersistentDataContainer.
   * static NamespacedKey namespacekey = namespacedKey.TYPE;
   */
  public NamespacedKey namespacekey;

  /**
   * The name of the attribute
   * static String name = "null";
   */
  public String name;

  public Attribute(NamespacedKey namespacekey, String name) {
    this.namespacekey = namespacekey;
    this.name = name;
  }

  /**
   * gives the name uppercased
   * @return the name in uppercase
   */
  public String nameUppercase() {
    return name.toUpperCase();
  }

  /**
   * get how many the Entity has Attributed
   * @param entity a living entity
   * @return the amount of attribute
   */
  public Integer get(@NotNull Entity entity) {
    if (
      entity
        .getPersistentDataContainer()
        .has(namespacekey, PersistentDataType.INTEGER)
    ) {
      debug(
        " attribute gett" +
        entity
          .getPersistentDataContainer()
          .get(namespacekey, PersistentDataType.INTEGER)
      );
      return entity
        .getPersistentDataContainer()
        .get(namespacekey, PersistentDataType.INTEGER);
    }
    debug(
      name +
      " not found in Entity: " +
      entity.getType().toString() +
      " " +
      entity.getName() +
      " " +
      entity.getUniqueId().toString()
    );
    set(entity, 0);
    return 0;
  }

  /**
   * get how many the Entity has Attributed
   * @param it an ItemStack
   * @return the amount of attribute
   */
  public Integer get(@NotNull ItemStack it) {
    NBTItem nbti = new NBTItem(it);
    if (nbti.hasTag(name)) {
      return nbti.getInteger(name);
    }
    return 0;
  }

  /**
   * Get the amount of attribute on the item
   * @param e a LivingEntity
   * @return an amount of attribute on the item
   */
  public Integer getItem(@NotNull LivingEntity e) {
    EntityEquipment equi = e.getEquipment();
    Integer attribute = 0;
    if (equi != null) {
      if (
        equi.getItemInMainHand().getType() != Material.AIR &&
        equi.getHelmet() != null
      ) {
        ItemStack item = equi.getItemInMainHand();
        attribute = attribute + get(item);
      }
      if (
        equi.getHelmet().getType() != Material.AIR && equi.getHelmet() != null
      ) {
        ItemStack item = equi.getHelmet();
        attribute = attribute + get(item);
      }
      if (
        equi.getChestplate().getType() != Material.AIR &&
        equi.getChestplate() != null
      ) {
        ItemStack item = equi.getChestplate();
        attribute = attribute + get(item);
      }
      if (
        equi.getChestplate().getType() != Material.AIR &&
        equi.getChestplate() != null
      ) {
        ItemStack item = equi.getChestplate();
        attribute = attribute + get(item);
      }
      if (
        equi.getLeggings().getType() != Material.AIR &&
        equi.getLeggings() != null
      ) {
        ItemStack item = equi.getLeggings();
        attribute = attribute + get(item);
      }
      if (
        equi.getBoots().getType() != Material.AIR && equi.getBoots() != null
      ) {
        ItemStack item = equi.getBoots();
        attribute = attribute + get(item);
      }
    }
    return attribute;
  }

  /**
   * set amount to the entity's attribute
   * @param entity a living entity
   * @param amount the amount you want to set
   */
  public void set(@NotNull Entity entity, @NotNull Integer amount) {
    entity
      .getPersistentDataContainer()
      .set(namespacekey, PersistentDataType.INTEGER, amount);
    debug("set " + name + " " + amount);
  }

  /**
   * add amount to the entity's attribute
   * @param entity a living entity
   * @param amount the amount you want to set
   */
  public void add(@NotNull Entity entity, @NotNull Integer amount) {
    set(entity, get(entity) + amount);
  }

  /**
   * remove amount to the entity's attribute
   * @param entity a living entity
   * @param amount the amount you want to set
   */
  public void remove(@NotNull Entity entity, @NotNull Integer amount) {
    set(entity, get(entity) - amount);
  }
}
