package me.unurled.raxen.manager.entity;

import static me.unurled.raxen.utils.Utils.debug;
import static me.unurled.raxen.utils.Utils.error;

import java.io.File;
import java.io.IOException;
import lombok.Getter;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.utils.libs.MongoDB;
import me.unurled.raxen.utils.libs.Mysql;
import org.bukkit.Bukkit;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

public class StorageManager {

  private static Raxen main;

  @Getter
  private static FileConfiguration config;

  @Getter
  private static MongoDB mongodb;

  @Getter
  private static Mysql mysql;

  public StorageManager(Raxen main) {
    this.main = main;
    this.config = main.getConfig();
    this.connect();
  }

  /**
   * Connects to the storage system (MONGO, YML, MYSQL (not implemented))
   */
  public static void connect() {
    String s = (String) config.get("storage");
    debug(main, s);
    mongodb = main.getManager().getLibsManager().getMongoDB();
    if (s.equalsIgnoreCase("MONGODB")) {
      mongodb.connect();
      debug(main, "'" + s + "'");
    } else if (s.equalsIgnoreCase("MYSQL")) {
      error(
        (Raxen) Bukkit.getPluginManager().getPlugin("Raxen"),
        "Currently not implemented, switching to YML"
      );
      debug(main, "'" + s + "'");
    } else {
      debug(main, "'" + s + "'");
    }
  }

  /**
   * disconnect mysql db
   */
  public static void disconnect() {
    mysql.disconnect();
  }

  /**
   * create a new yml for the player
   * @param player
   * @return the created yml
   */
  public static @NotNull FileConfiguration createYml(@NotNull Player player) {
    File customFile;
    FileConfiguration customConfig;
    customFile =
      new File(
        main.getDataFolder() + "/playerInfo/" + player.getUniqueId(),
        "/playerInfo.yml"
      );

    if (!customFile.exists()) {
      customFile.getParentFile().mkdirs();
      try {
        customFile.createNewFile();
      } catch (IOException e) {
        error(
          (Raxen) Bukkit.getPluginManager().getPlugin("Raxen"),
          "Error in Storage Manager saving new File."
        );
        e.printStackTrace();
      }
    }

    customConfig = new YamlConfiguration();
    try {
      customConfig.load(customFile);
    } catch (IOException | InvalidConfigurationException e) {
      e.printStackTrace();
    }
    if (customConfig.get("uuid") == null) {
      customConfig.set("uuid", player.getUniqueId().toString());
      customConfig.set("name", player.getName());
      customConfig.set("health", 100);
      customConfig.set("itemHealth", 0);
      customConfig.set("defense", 50);
      customConfig.set("itemDefense", 0);
      customConfig.set("speed", 100);
      customConfig.set("itemSpeed", 0);
      customConfig.set("strength", 100);
      customConfig.set("itemDmg", 0);
      customConfig.set("inv", "");
      customConfig.set("ec", "");
      try {
        customConfig.save(customFile);
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
    return customConfig;
  }

  /**
   * create a config yaml with the file
   * @param file
   * @return a file configuration
   */
  public static @NotNull FileConfiguration createYml(@NotNull File file) {
    FileConfiguration customConfig;

    if (!file.exists()) {
      file.getParentFile().mkdirs();
      try {
        file.createNewFile();
      } catch (IOException e) {
        error(
          (Raxen) Bukkit.getPluginManager().getPlugin("Raxen"),
          "Error in Storage Manager saving new File."
        );
        e.printStackTrace();
      }
    }
    customConfig = new YamlConfiguration();
    try {
      customConfig.load(file);
    } catch (IOException | InvalidConfigurationException e) {
      e.printStackTrace();
    }
    return customConfig;
  }

  /**
   * create a config in the path provided
   * @param path
   * @return the created config file
   */
  public static @NotNull FileConfiguration createYml(String path) {
    File customFile;
    FileConfiguration customConfig;
    customFile = new File(path);
    if (!customFile.exists()) {
      customFile.getParentFile().mkdirs();
      try {
        customFile.createNewFile();
      } catch (IOException e) {
        error(
          (Raxen) Bukkit.getPluginManager().getPlugin("Raxen"),
          "Error in Storage Manager saving new File."
        );
        e.printStackTrace();
      }
    }
    customConfig = new YamlConfiguration();
    try {
      customConfig.load(customFile);
    } catch (IOException | InvalidConfigurationException e) {
      e.printStackTrace();
    }
    return customConfig;
  }
}
